@@ -141,22 +141,21 @@ dialog_client_get_coverage(Client *c, int *x, int *y, int *w, int *h)
 void
 dialog_client_move_resize(Client *c)
 {
-   int frm_size     = dialog_client_title_height(c);
-   int offset_south = 0, offset_west = 0, offset_east = 0;
+  Wm *w = c->wm;
+  int frm_size     = dialog_client_title_height(c);
+  int offset_south = 0, offset_west = 0, offset_east = 0;
 
-   dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
+  dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
-   base_client_move_resize(c);
+  base_client_move_resize(c);
 
-   XMoveResizeWindow(c->wm->dpy, c->window, 
-		     offset_west,
-		     frm_size,
-		     c->width, c->height);
+  XMoveResizeWindow(w->dpy, c->window, offset_west, frm_size,
+		    c->width, c->height);
 
 #ifndef USE_COMPOSITE
-   if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
+   if (w->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
      {
-       XMoveWindow(c->wm->dpy, 
+       XMoveWindow(w->dpy, 
 		   c->window, 
 		   c->x, 
 		   c->y);
@@ -164,7 +163,7 @@ dialog_client_move_resize(Client *c)
    else
 #endif
      {
-       XMoveResizeWindow(c->wm->dpy, 
+       XMoveResizeWindow(w->dpy, 
 			 c->frame, 
 			 c->x - offset_west, 
 			 c->y - frm_size, 
@@ -176,6 +175,13 @@ dialog_client_move_resize(Client *c)
    if (!(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
      client_decor_frames_move_resize(c, offset_west, offset_east, 
 				     frm_size, offset_south);
+
+   if (c->win_modal_blocker) 
+     {
+       XMoveResizeWindow(w->dpy, c->win_modal_blocker, 
+			 0, 0, w->dpy_width, w->dpy_height);
+     }
+
 }
 
 int
@@ -312,7 +318,7 @@ dialog_client_reparent(Client *c)
   attr.border_pixel      = 0;
   attr.event_mask        = ChildMask|ButtonPressMask|ExposureMask;
   
-  attr.colormap          = c->cmap; 
+  attr.colormap          = c->cmap; /* Not used - needed ? */
   
   dbg("%s() want lowlight : wm:%i , client:%i\n", __func__,
       c->wm->config->dialog_shade, (c->flags & CLIENT_IS_MODAL_FLAG));
@@ -345,7 +351,7 @@ dialog_client_reparent(Client *c)
 				     CopyFromParent,
 #endif
 				     CWOverrideRedirect|CWEventMask
-				     |CWBackPixel|CWBorderPixel|CWColormap, 
+				     |CWBackPixel|CWBorderPixel/*|CWColormap*/, 
 				     &attr);
      }
 
@@ -358,6 +364,28 @@ dialog_client_reparent(Client *c)
 
      }
 
+   if (c->flags & CLIENT_IS_MODAL_FLAG
+       && c->trans == NULL )	/* modal for device. XXX check recursive ? */
+     {
+       /* Create an InputOnly fullscreen window to aid in making 
+	* modal dialogs *really* modal to the whole display by
+	* block button events. 
+       */
+       c->win_modal_blocker = XCreateWindow(w->dpy, 
+					    w->root, 
+					    0, 0,
+					    w->dpy_width,
+					    w->dpy_height,
+					    0,
+					    CopyFromParent,
+					    InputOnly,  
+					    CopyFromParent,
+					    CWOverrideRedirect|CWEventMask,
+					    &attr);
+       XMapWindow(w->dpy, c->win_modal_blocker);
+       w->stack_n_items++;
+     }
+
    XClearWindow(w->dpy, c->frame);
    
    XSetWindowBorderWidth(w->dpy, c->window, 0);
@@ -370,6 +398,8 @@ dialog_client_reparent(Client *c)
    if (c->frame != c->window)
      XReparentWindow(w->dpy, c->window, c->frame, 
 		     offset_west, offset_north);
+
+
 }
 
 /*  Padding between dialog borders and area available */
@@ -810,55 +840,6 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 
   dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
-  if (w->config->dialog_stratergy == WM_DIALOGS_STRATERGY_STATIC)
-    {
-      /* For static undraggble/stack fixed dialog we simply  
-       * hide the dialog when titlebar is clicked on. 
-       *
-       * TODO: Ideally this code should go in drag loop. but
-       *       it seems the servergrab() stops underlying app
-       *       from repainting itself. Need to look more into 
-       *       this.
-       *       Is it safe to simply remove the server grab ?
-       *
-       */
-      if (c->flags & CLIENT_HAS_URGENCY_FLAG)
-	return; 		/* No Effect for message wins */
-
-      if (XGrabPointer(w->dpy, w->root, False,
-		       ButtonPressMask|ButtonReleaseMask,
-		       GrabModeAsync,
-		       GrabModeAsync, 
-		       None, w->curs, CurrentTime) == GrabSuccess)
-	{
-	  XUnmapWindow(w->dpy, c->frame);
-	  _draw_outline(c, c->x - offset_west, c->y - offset_north,
-			c->width + offset_west + offset_east,
-			c->height + offset_north + offset_south);
-	  
-	  XFlush(w->dpy);
-	  
-	  for (;;) 
-	    {
-	      XEvent xev;
-	      XMaskEvent(w->dpy,ButtonReleaseMask|ButtonPressMask, &xev);
-	      if (xev.type == ButtonRelease)
-		{
-		  _draw_outline(c, c->x - offset_west, c->y - offset_north,
-				c->width + offset_west + offset_east,
-				c->height + offset_north + offset_south);
-		  
-		  XMapWindow(w->dpy, c->frame);
-		  break;
-		}
-	    }
-	  
-	  XUngrabPointer(w->dpy, CurrentTime); 
-	  XSync(w->dpy, False);
-	}
-      return;
-    }
-
   switch (client_button_do_ops(c, e, FRAME_DIALOG, 
 			       c->width + offset_east + offset_west, 
 			       offset_north))
@@ -874,10 +855,62 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
       case -1: 		 /* Cancelled  */
 	 break;
       case 0:
+	/* Not on button */
 	if (w->config->dialog_stratergy == WM_DIALOGS_STRATERGY_STATIC)
-	  break; 		/* no dragging, raising */
-	dialog_client_drag(c);  /* Not on button */
-	 break;
+	  {
+	    /* For static undraggble/stack fixed dialog we simply  
+	     * hide the dialog when titlebar is clicked on. 
+	     *
+	     * TODO: Ideally this code should go in drag loop. but
+	     *       it seems the servergrab() stops underlying app
+	     *       from repainting itself. Need to look more into 
+	     *       this.
+	     *       Is it safe to simply remove the server grab ?
+	     *
+	     */
+	    if (c->flags & CLIENT_HAS_URGENCY_FLAG)
+	      return; 		/* No Effect for message wins */
+	    
+	    if (e->window != c->frames_decor[NORTH])
+	      return;
+	    
+	    if (XGrabPointer(w->dpy, w->root, False,
+			     ButtonPressMask|ButtonReleaseMask,
+			     GrabModeAsync,
+			     GrabModeAsync, 
+			     None, w->curs, CurrentTime) == GrabSuccess)
+	      {
+		XUnmapWindow(w->dpy, c->frame);
+		_draw_outline(c, c->x - offset_west, c->y - offset_north,
+			      c->width + offset_west + offset_east,
+			      c->height + offset_north + offset_south);
+		
+		XFlush(w->dpy);
+		
+		for (;;) 
+		  {
+		    XEvent xev;
+		    XMaskEvent(w->dpy,ButtonReleaseMask|ButtonPressMask, &xev);
+		    if (xev.type == ButtonRelease)
+		      {
+			_draw_outline(c, c->x - offset_west, 
+				      c->y - offset_north,
+				      c->width + offset_west + offset_east,
+				      c->height + offset_north + offset_south);
+			XMapWindow(w->dpy, c->frame);
+			break;
+		      }
+		  }
+		
+		XUngrabPointer(w->dpy, CurrentTime); 
+		XSync(w->dpy, False);
+	      }
+
+	    return;
+	  }
+
+	dialog_client_drag(c); 
+	break;
    }
 }
 
@@ -1054,13 +1087,18 @@ void
 dialog_client_iconize(Client *c)
 {
   Wm     *w = c->wm; 
-  Client *d = NULL;
+  Client *d = NULL, *p = NULL;
 
   client_set_state(c, IconicState);
   c->flags |= CLIENT_IS_MINIMIZED;
   c->mapped = False;
   XUnmapWindow(w->dpy, c->frame); 
-  
+
+  /* Make sure any transients get iconized too */  
+  stack_enumerate(w, p)
+    if (p->trans == c)
+      p->iconize(p);
+
   if ((d = dialog_client_set_focus_next(d)) != NULL)
     wm_activate_client(d);
 }
@@ -1068,33 +1106,19 @@ dialog_client_iconize(Client *c)
 void 
 dialog_client_destroy(Client *c)
 {
-  Wm     *w = c->wm; 
   Client *d = NULL;
 
-#ifdef MSG_Q
-  int was_msg = 0;
-
-  if (c->flags & CLIENT_IS_MESSAGE_DIALOG)
-    {
-      if (w->msg_win_queue_head 
-	  && w->msg_win_queue_head->win == c->window);
-      was_msg = 1;
-    }
-#endif
-
-
   /* Focus the saved next or return a likely candidate if none found */
   d = dialog_client_set_focus_next(c);
 
-  base_client_destroy(c);
+  if (c->win_modal_blocker)
+    {
+      Wm *w = c->wm;
+      XDestroyWindow(w->dpy, c->win_modal_blocker);
+      w->stack_n_items--;
+    }
 
-#ifdef MSG_Q
-if (was_msg)
-  {
-    dbg("%s() was message poping queue\n", __func__);
-    wm_msg_win_queue_pop(w);
-  }
-#endif
+  base_client_destroy(c);
 
   /* 
    *  We call activate_client mainly to figure out what to focus next.
@@ -1103,5 +1127,4 @@ if (was_msg)
    */
   if (d) 
       wm_activate_client(d);
-
 }
